import { __read, __spread } from "tslib";
import { isInHierarchy } from '@antv/dw-util';
import * as DWAnalyzer from '@antv/dw-analyzer';
import { isNil } from 'lodash';
export function getChartTypeSpec(chartType, dataProps, dataset) {
    switch (chartType) {
        case 'pie_chart':
            return pie_chart(dataProps);
        case 'donut_chart':
            return donut_chart(dataProps);
        case 'line_chart':
            return line_chart(dataProps);
        case 'step_line_chart':
            return step_line_chart(dataProps);
        case 'area_chart':
            return area_chart(dataProps);
        case 'stacked_area_chart':
            return stacked_area_chart(dataProps);
        case 'percent_stacked_area_chart':
            return percent_stacked_area_chart(dataProps);
        case 'bar_chart':
            return bar_chart(dataProps);
        case 'grouped_bar_chart':
            return grouped_bar_chart(dataProps);
        case 'stacked_bar_chart':
            return stacked_bar_chart(dataProps);
        case 'percent_stacked_bar_chart':
            return percent_stacked_bar_chart(dataProps);
        case 'column_chart':
            return column_chart(dataProps);
        case 'grouped_column_chart':
            return grouped_column_chart(dataProps, dataset);
        case 'stacked_column_chart':
            return stacked_column_chart(dataProps, dataset);
        case 'percent_stacked_column_chart':
            return percent_stacked_column_chart(dataProps, dataset);
        // https://github.com/vega/vega-lite/issues/3805
        // case 'radar_chart':
        //   return radar_chart(dataProps);
        case 'scatter_plot':
            return scatter_plot(dataProps);
        case 'bubble_chart':
            return bubble_chart(dataProps);
        case 'histogram':
            return histogram(dataProps);
        case 'heatmap':
            return heatmap(dataProps);
        case 'kpi_panel':
            return kpi_panel();
        case 'table':
            return table(dataProps);
        default:
            return null;
    }
}
function hasSubset(array1, array2) {
    return array2.every(function (e) { return array1.includes(e); });
}
function intersects(array1, array2) {
    return array2.some(function (e) { return array1.includes(e); });
}
function compare(f1, f2) {
    if (isNil(f1.distinct) || isNil(f2.distinct)) {
        if (f1.distinct < f2.distinct) {
            return 1;
        }
        else if (f1.distinct > f2.distinct) {
            return -1;
        }
        else {
            return 0;
        }
    }
    else {
        return 0;
    }
}
function LOM2EncodingType(lom) {
    switch (lom) {
        case 'Nominal':
            return 'nominal';
        case 'Ordinal':
            return 'ordinal';
        case 'Interval':
            return 'quantitative';
        case 'Time':
            return 'temporal';
        case 'Continuous':
            return 'quantitative';
        case 'Discrete':
            return 'nominal';
        default:
            return 'nominal';
    }
}
/* pie_chart & donut_chart */
function splitAngleColor(dataProps) {
    var field4Color = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Nominal']); });
    var field4Angle = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    return [field4Color, field4Angle];
}
function pie_chart(dataProps) {
    var _a = __read(splitAngleColor(dataProps), 2), field4Color = _a[0], field4Angle = _a[1];
    if (!field4Angle || !field4Color)
        return null;
    var spec = {
        mark: { type: 'arc' },
        encoding: {
            theta: { field: field4Angle.name, type: 'quantitative' },
            color: { field: field4Color.name, type: 'nominal' },
        },
    };
    return spec;
}
function donut_chart(dataProps) {
    var _a = __read(splitAngleColor(dataProps), 2), field4Color = _a[0], field4Angle = _a[1];
    if (!field4Angle || !field4Color)
        return null;
    return {
        mark: { type: 'arc', innerRadius: 50 },
        encoding: {
            theta: { field: field4Angle.name, type: 'quantitative' },
            color: { field: field4Color.name, type: 'nominal' },
        },
    };
}
/* line_chart & step_line_chart */
function splitLineXY(dataProps) {
    var field4X = dataProps.find(function (field) { return intersects(field.levelOfMeasurements, ['Time', 'Ordinal']); });
    var field4Y = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    var field4Color = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Nominal']); });
    return [field4X, field4Y, field4Color];
}
function line_chart(dataProps) {
    var _a = __read(splitLineXY(dataProps), 3), field4X = _a[0], field4Y = _a[1], field4Color = _a[2];
    if (!field4X || !field4Y)
        return null;
    var spec = {
        mark: { type: 'line' },
        encoding: {
            x: { field: field4X.name, type: LOM2EncodingType(field4X.levelOfMeasurements[0]) },
            y: { field: field4Y.name, type: 'quantitative' },
        },
    };
    if (field4Color) {
        spec.encoding.color = { field: field4Color.name, type: 'nominal' };
    }
    return spec;
}
function step_line_chart(dataProps) {
    var _a = __read(splitLineXY(dataProps), 3), field4X = _a[0], field4Y = _a[1], field4Color = _a[2];
    if (!field4X || !field4Y)
        return null;
    // interpolate can be step | 'step-after' | 'step-before'
    var spec = {
        mark: { type: 'line', interpolate: 'step' },
        encoding: {
            x: { field: field4X.name, type: LOM2EncodingType(field4X.levelOfMeasurements[0]) },
            y: { field: field4Y.name, type: 'quantitative' },
        },
    };
    if (field4Color) {
        spec.encoding.color = { field: field4Color.name, type: 'nominal' };
    }
    return spec;
}
function area_chart(dataProps) {
    var field4X = dataProps.find(function (field) { return intersects(field.levelOfMeasurements, ['Time', 'Ordinal']); });
    var field4Y = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    if (!field4X || !field4Y)
        return null;
    var spec = {
        mark: { type: 'area' },
        encoding: {
            x: { field: field4X.name, type: LOM2EncodingType(field4X.levelOfMeasurements[0]) },
            y: { field: field4Y.name, type: 'quantitative' },
        },
    };
    return spec;
}
function splitAreaXYSeries(dataProps) {
    var field4X = dataProps.find(function (field) { return intersects(field.levelOfMeasurements, ['Time', 'Ordinal']); });
    var field4Series = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Nominal']); });
    var field4Y = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    return [field4X, field4Y, field4Series];
}
function stacked_area_chart(dataProps) {
    var _a = __read(splitAreaXYSeries(dataProps), 3), field4X = _a[0], field4Y = _a[1], field4Series = _a[2];
    if (!field4X || !field4Y || !field4Series)
        return null;
    var spec = {
        mark: { type: 'area' },
        encoding: {
            x: { field: field4X.name, type: LOM2EncodingType(field4X.levelOfMeasurements[0]) },
            y: { field: field4Y.name, type: 'quantitative', stack: 'zero' },
            color: { field: field4Series.name, type: 'nominal' },
        },
    };
    return spec;
}
function percent_stacked_area_chart(dataProps) {
    var _a = __read(splitAreaXYSeries(dataProps), 3), field4X = _a[0], field4Y = _a[1], field4Series = _a[2];
    if (!field4X || !field4Y || !field4Series)
        return null;
    var spec = {
        mark: { type: 'area' },
        encoding: {
            x: { field: field4X.name, type: LOM2EncodingType(field4X.levelOfMeasurements[0]) },
            y: { field: field4Y.name, type: 'quantitative', stack: 'normalize' },
            color: { field: field4Series.name, type: 'nominal' },
        },
    };
    return spec;
}
function bar_chart(dataProps) {
    var nominalFields = dataProps.filter(function (field) { return hasSubset(field.levelOfMeasurements, ['Nominal']); });
    var sortedNominalFields = nominalFields.sort(compare);
    var field4Y = sortedNominalFields[0];
    var field4Color = sortedNominalFields[1];
    var field4X = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    if (!field4X || !field4Y)
        return null;
    var spec = {
        mark: { type: 'bar' },
        encoding: {
            x: { field: field4X.name, type: 'quantitative' },
            y: { field: field4Y.name, type: 'nominal' },
        },
    };
    if (field4Color) {
        spec.encoding.color = { field: field4Color.name, type: 'nominal' };
    }
    return spec;
}
function column_chart(dataProps) {
    var nominalFields = dataProps.filter(function (field) { return hasSubset(field.levelOfMeasurements, ['Nominal']); });
    var sortedNominalFields = nominalFields.sort(compare);
    var field4X = sortedNominalFields[0];
    var field4Color = sortedNominalFields[1];
    var field4Y = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    if (!field4X || !field4Y)
        return null;
    var spec = {
        mark: { type: 'bar' },
        encoding: {
            x: { field: field4X.name, type: 'nominal' },
            y: { field: field4Y.name, type: 'quantitative' },
        },
    };
    if (field4Color) {
        spec.encoding.color = { field: field4Color.name, type: 'nominal' };
    }
    return spec;
}
/** grouped_column_chart stacked_column_chart percent_stacked_column_chart */
function splitColumnXYSeries(dataProps, dataset) {
    var _a, _b;
    var nominalFields = dataProps.filter(function (field) { return hasSubset(field.levelOfMeasurements, ['Nominal']); });
    var sortedNominalFields = nominalFields.sort(compare);
    var field4X;
    var Field4Series;
    if (dataset && isInHierarchy((_a = sortedNominalFields[1]) === null || _a === void 0 ? void 0 : _a.samples, (_b = sortedNominalFields[0]) === null || _b === void 0 ? void 0 : _b.samples)) {
        field4X = sortedNominalFields[1];
        Field4Series = sortedNominalFields[0];
    }
    else {
        field4X = sortedNominalFields[0];
        Field4Series = sortedNominalFields[1];
    }
    var field4Y = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    return [field4X, field4Y, Field4Series];
}
function grouped_column_chart(dataProps, dataset) {
    var _a = __read(splitColumnXYSeries(dataProps, dataset), 3), field4X = _a[0], field4Y = _a[1], Field4Series = _a[2];
    if (!field4X || !field4Y || !Field4Series)
        return null;
    var spec = {
        mark: { type: 'bar' },
        encoding: {
            x: { field: Field4Series.name, type: 'nominal' },
            y: { field: field4Y.name, type: 'quantitative' },
            column: { field: field4X.name, type: 'nominal' },
            color: { field: Field4Series.name, type: 'nominal' },
        },
    };
    return spec;
}
function stacked_column_chart(dataProps, dataset) {
    var _a = __read(splitColumnXYSeries(dataProps, dataset), 3), field4X = _a[0], field4Y = _a[1], Field4Series = _a[2];
    if (!field4X || !field4Y || !Field4Series)
        return null;
    var spec = {
        mark: { type: 'bar' },
        encoding: {
            x: { field: field4X.name, type: 'nominal' },
            y: { field: field4Y.name, type: 'quantitative', stack: 'zero' },
            color: { field: Field4Series.name, type: 'nominal' },
        },
    };
    return spec;
}
function percent_stacked_column_chart(dataProps, dataset) {
    var _a = __read(splitColumnXYSeries(dataProps, dataset), 3), field4X = _a[0], field4Y = _a[1], Field4Series = _a[2];
    if (!field4X || !field4Y || !Field4Series)
        return null;
    var spec = {
        mark: { type: 'bar' },
        encoding: {
            x: { field: field4X.name, type: 'nominal' },
            y: { field: field4Y.name, type: 'quantitative', stack: 'normalize' },
            color: { field: Field4Series.name, type: 'nominal' },
        },
    };
    return spec;
}
/** grouped_bar_chart stacked_bar_chart percent_stacked_bar_chart */
function splitBarXYSeries(dataProps) {
    var nominalFields = dataProps.filter(function (field) { return hasSubset(field.levelOfMeasurements, ['Nominal']); });
    var sortedNominalFields = nominalFields.sort(compare);
    var field4Y = sortedNominalFields[0];
    var Field4Series = sortedNominalFields[1];
    var field4X = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    return [field4X, field4Y, Field4Series];
}
function grouped_bar_chart(dataProps) {
    var _a = __read(splitBarXYSeries(dataProps), 3), field4X = _a[0], field4Y = _a[1], Field4Series = _a[2];
    if (!field4X || !field4Y || !Field4Series)
        return null;
    var spec = {
        mark: { type: 'bar' },
        encoding: {
            x: { field: field4X.name, type: 'quantitative' },
            y: { field: Field4Series.name, type: 'nominal' },
            row: { field: field4Y.name, type: 'nominal' },
            color: { field: Field4Series.name, type: 'nominal' },
        },
    };
    return spec;
}
function stacked_bar_chart(dataProps) {
    var _a = __read(splitBarXYSeries(dataProps), 3), field4X = _a[0], field4Y = _a[1], Field4Series = _a[2];
    if (!field4X || !field4Y || !Field4Series)
        return null;
    var spec = {
        mark: { type: 'bar' },
        encoding: {
            x: { field: field4X.name, type: 'quantitative', stack: 'zero' },
            y: { field: field4Y.name, type: 'nominal' },
            color: { field: Field4Series.name, type: 'nominal' },
        },
    };
    return spec;
}
function percent_stacked_bar_chart(dataProps) {
    var _a = __read(splitBarXYSeries(dataProps), 3), field4X = _a[0], field4Y = _a[1], Field4Series = _a[2];
    if (!field4X || !field4Y || !Field4Series)
        return null;
    var spec = {
        mark: { type: 'bar' },
        encoding: {
            x: { field: field4X.name, type: 'quantitative', stack: 'normalize' },
            y: { field: field4Y.name, type: 'nominal' },
            color: { field: Field4Series.name, type: 'nominal' },
        },
    };
    return spec;
}
function scatter_plot(dataProps) {
    var intervalFields = dataProps.filter(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    var sortedIntervalFields = intervalFields.sort(compare);
    var field4X = sortedIntervalFields[0];
    var field4Y = sortedIntervalFields[1];
    var field4Color = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Nominal']); });
    if (!field4X || !field4Y)
        return null;
    var spec = {
        mark: { type: 'circle' },
        encoding: {
            x: { field: field4X.name, type: 'quantitative' },
            y: { field: field4Y.name, type: 'quantitative' },
        },
    };
    if (field4Color) {
        spec.encoding.color = { field: field4Color.name, type: 'nominal' };
    }
    return spec;
}
function bubble_chart(dataProps) {
    var intervalFields = dataProps.filter(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    var triple = {
        x: intervalFields[0],
        y: intervalFields[1],
        corr: 0,
        size: intervalFields[2],
    };
    var _loop_1 = function (i) {
        var _loop_2 = function (j) {
            var p = DWAnalyzer.pearson(intervalFields[i], intervalFields[j]);
            if (Math.abs(p) > triple.corr) {
                triple.x = intervalFields[i];
                triple.y = intervalFields[j];
                triple.corr = p;
                triple.size = intervalFields[__spread(Array(intervalFields.length).keys()).find(function (e) { return e !== i && e !== j; }) || 0];
            }
        };
        for (var j = i + 1; j < intervalFields.length; j++) {
            _loop_2(j);
        }
    };
    for (var i = 0; i < intervalFields.length; i++) {
        _loop_1(i);
    }
    var field4X = triple.x;
    var field4Y = triple.y;
    var field4Size = triple.size;
    var field4Color = dataProps.find(function (field) { return intersects(field.levelOfMeasurements, ['Nominal']); });
    '';
    if (!field4X || !field4Y || !field4Size || !field4Color)
        return null;
    var spec = {
        mark: { type: 'circle' },
        encoding: {
            x: { field: field4X.name, type: 'quantitative' },
            y: { field: field4Y.name, type: 'quantitative' },
            color: { field: field4Color.name, type: 'nominal' },
            size: { field: field4Size.name, type: 'quantitative' },
        },
    };
    return spec;
}
function histogram(dataProps) {
    var field = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    if (!field)
        return null;
    var spec = {
        mark: { type: 'bar' },
        encoding: {
            x: { field: field.name, bin: true },
            y: { aggregate: 'count' },
        },
    };
    return spec;
}
function heatmap(dataProps) {
    var axisFields = dataProps.filter(function (field) { return intersects(field.levelOfMeasurements, ['Nominal', 'Ordinal']); });
    var sortedFields = axisFields.sort(compare);
    var field4X = sortedFields[0];
    var field4Y = sortedFields[1];
    var field4Color = dataProps.find(function (field) { return hasSubset(field.levelOfMeasurements, ['Interval']); });
    if (!field4X || !field4Y || !field4Color)
        return null;
    var spec = {
        mark: { type: 'rect' },
        encoding: {
            x: { field: field4X.name, type: LOM2EncodingType(field4X.levelOfMeasurements[0]) },
            y: { field: field4Y.name, type: LOM2EncodingType(field4Y.levelOfMeasurements[0]) },
            color: { field: field4Color.name, type: 'quantitative' },
        },
    };
    return spec;
}
function kpi_panel() {
    // TODO 指标卡暂不做更细致的配置，只支持基本的数值显示
    return null;
}
function table(dataProps) {
    var values = [];
    var rows = [];
    for (var i = 0; i < dataProps.length; i++) {
        var field = dataProps[i];
        if (intersects(field.levelOfMeasurements, ['Interval', 'Continuous', 'Discrete'])) {
            values.push(field.name);
        }
        else {
            rows.push(field.name);
        }
    }
    return {
        rows: rows,
        values: values,
        columns: [],
    };
}
